#pragma once
#include "mbed.h"
#include "FATFileSystem.h"
#include "USBMSD.h"
#include "USBMSD2.h"
#include "StorageInterface.h"
#include "mystring.h"

class LocalStorage : public FATFileSystem {
public:
    LocalStorage(StorageInterface* storage);
    virtual int disk_read(uint8_t * buffer, uint64_t sector) {
        return _storage->storage_read(buffer, sector);
    }
    virtual int disk_write(const uint8_t * buffer, uint64_t sector) {
        return _storage->storage_write(buffer, sector);
    }
    virtual uint64_t disk_sectors() {
        return _storage->storage_sectors();
    }
    static bool find_bin(mystring& filename);
private:
    StorageInterface* _storage;
};

class USBStorage : public USBMSD {
public:
    USBStorage(StorageInterface* storage): _storage(storage) {
        connect();
    }
    virtual int disk_read(uint8_t * data, uint64_t block) {
        return _storage->storage_read(data, block);
    }
    virtual int disk_write(const uint8_t * data, uint64_t block) {
        return _storage->storage_write(data, block);
    }
    virtual int disk_initialize() { return _storage->storage_initialize(); }
    virtual uint64_t disk_sectors() { return _storage->storage_sectors(); }
    virtual uint64_t disk_size() { return _storage->storage_size(); }
    virtual int disk_status() { return _storage->storage_status(); }

private:
    StorageInterface* _storage;
};

class USBStorage2 : public USBMSD2 {
public:
    USBStorage2(StorageInterface* storage): _storage(storage) {
        connect();
    }
    virtual int disk_read(uint8_t * data, uint64_t block) {
        return _storage->storage_read(data, block);
    }
    virtual int disk_write(const uint8_t * data, uint64_t block) {
        return _storage->storage_write(data, block);
    }
    virtual int disk_initialize() { return _storage->storage_initialize(); }
    virtual uint64_t disk_sectors() { return _storage->storage_sectors(); }
    virtual uint64_t disk_size() { return _storage->storage_size(); }
    virtual int disk_status() { return _storage->storage_status(); }

private:
    StorageInterface* _storage;
};
